<?php

namespace Drupal\layout_builder_base\Plugin\Layout;

use Drupal\Core\Form\FormStateInterface;
use Drupal\file\Entity\File;
use Drupal\layout_builder\Plugin\Layout\MultiWidthLayoutBase;

/**
 * Configurable layout plugin class.
 *
 * @internal
 *   Plugin classes are internal.
 */
abstract class BaseLayoutBase extends MultiWidthLayoutBase {

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    $config = [
      'column_gap' => $this->getDefaultColumnGap(),
      'row_gap' => $this->getDefaultRowGap(),
      'column_width' => $this->getDefaultColumnWidth(),
      'column_breakpoint' => $this->getDefaultColumnBreakpoint(),
      'background' => $this->getDefaultBackground(),
      'background_image' => $this->getDefaultImageBackground(),
      'background_attachment' => $this->getDefaultBackgroundAttachment(),
      'background_position' => $this->getDefaultBackgroundPosition(),
      'background_size' => $this->getDefaultBackgroundSize(),
      'background_overlay' => $this->getDefaultBackgroundOverlay(),
      'equal_top_bottom_margin' => $this->getDefaultEqualTopBottomMargin(),
      'equal_left_right_margin' => $this->getDefaultEqualLeftRightMargin(),
      'top_margin' => $this->getDefaultTopMargin(),
      'right_margin' => $this->getDefaultRightMargin(),
      'bottom_margin' => $this->getDefaultBottomMargin(),
      'left_margin' => $this->getDefaultLeftMargin(),
      'equal_top_bottom_padding' => $this->getDefaultEqualTopBottomPadding(),
      'equal_left_right_padding' => $this->getDefaultEqualLeftRightPadding(),
      'top_padding' => $this->getDefaultTopPadding(),
      'right_padding' => $this->getDefaultRightPadding(),
      'bottom_padding' => $this->getDefaultBottomPadding(),
      'left_padding' => $this->getDefaultLeftPadding(),
      'container' => $this->getDefaultContainer(),
      'color' => $this->getDefaultColor(),
      'modifier' => $this->getDefaultModifiers(),
    ];

    return array_merge(parent::defaultConfiguration(), $config);
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    if (!empty($this->getWidthOptions())) {
      $form = parent::buildConfigurationForm($form, $form_state);
    }

    if ($this->isMultipleColumnsLayout()) {
      if (!empty($this->getColumnGapOptions())) {
        $form['column_gap'] = [
          '#type' => 'select',
          '#title' => $this->t('Column gap'),
          '#default_value' => $this->configuration['column_gap'],
          '#options' => $this->getColumnGapOptions(),
          '#description' => $this->t('Choose the column gap for this layout.'),
        ];
      }

      if (!empty($this->getRowGapOptions())) {
        $form['row_gap'] = [
          '#type' => 'select',
          '#title' => $this->t('Row gap'),
          '#default_value' => $this->configuration['row_gap'],
          '#options' => $this->getRowGapOptions(),
          '#description' => $this->t('Choose the row gap for this layout.'),
        ];
      }

      if (!empty($this->getColumnBreakpointOptions())) {
        $form['column_breakpoint'] = [
          '#type' => 'select',
          '#title' => $this->t('Column breakpoint'),
          '#default_value' => $this->configuration['column_breakpoint'],
          '#options' => $this->getColumnBreakpointOptions(),
          '#description' => $this->t('Choose the column breakpoint where it comes back to one column for this layout.'),
        ];
      }

      if (!empty($this->getColumnWidthOptions())) {
        $form['column_width'] = [
          '#type' => 'select',
          '#title' => $this->t('Column width'),
          '#default_value' => $this->configuration['column_width'],
          '#options' => $this->getColumnWidthOptions(),
          '#description' => $this->t('Choose the column width for this layout.'),
        ];
      }
    }

    if ($this->enableImageBackground()) {
      $form['background_image'] = [
        '#type' => 'managed_file',
        '#title' => $this->t('Background image'),
        '#default_value' => (!empty($this->configuration['background_image'])) ? [$this->configuration['background_image']] : '',
        '#description' => $this->t('Choose a background image for this layout. This feature is using inline CSS.'),
        '#upload_validators' => [
          'file_validate_is_image' => [],
          'file_validate_extensions' => ['gif png jpg jpeg'],
        ],
        '#upload_location' => 'public://layout_background/',
      ];
    }

    if (!empty($this->getBackgroundOptions())) {
      $form['background'] = [
        '#type' => 'select',
        '#title' => $this->t('Background'),
        '#default_value' => $this->configuration['background'],
        '#options' => $this->getBackgroundOptions(),
        '#description' => $this->t('Choose the background for this layout.'),
      ];
    }

    if (!empty($this->getBackgroundAttachmentOptions())) {
      $form['background_attachment'] = [
        '#type' => 'select',
        '#title' => $this->t('Background Attachment'),
        '#default_value' => $this->configuration['background_attachment'],
        '#options' => $this->getBackgroundAttachmentOptions(),
        '#description' => $this->t('Choose the background attachment for this layout.'),
      ];
    }

    if (!empty($this->getBackgroundPositionOptions())) {
      $form['background_position'] = [
        '#type' => 'select',
        '#title' => $this->t('Background Position'),
        '#default_value' => $this->configuration['background_position'],
        '#options' => $this->getBackgroundPositionOptions(),
        '#description' => $this->t('Choose the background position for this layout.'),
      ];
    }

    if (!empty($this->getBackgroundSizeOptions())) {
      $form['background_size'] = [
        '#type' => 'select',
        '#title' => $this->t('Background Size'),
        '#default_value' => $this->configuration['background_size'],
        '#options' => $this->getBackgroundSizeOptions(),
        '#description' => $this->t('Choose the background size for this layout.'),
      ];
    }

    if (!empty($this->getBackgroundOverlayOptions())) {
      $form['background_overlay'] = [
        '#type' => 'select',
        '#title' => $this->t('Background Overlay'),
        '#default_value' => $this->configuration['background_overlay'],
        '#options' => $this->getBackgroundOverlayOptions(),
        '#description' => $this->t('Choose the background overlay for this layout.'),
      ];
    }

    if (!empty($this->getEqualTopBottomMarginsOptions())) {
      $form['equal_top_bottom_margin'] = [
        '#type' => 'select',
        '#title' => $this->t('Top Bottom Margins'),
        '#default_value' => $this->configuration['equal_top_bottom_margin'],
        '#options' => $this->getEqualTopBottomMarginsOptions(),
        '#description' => $this->t('Choose the top/bottom margins for this layout.'),
      ];
    }

    if (!empty($this->getEqualLeftRightMarginsOptions())) {
      $form['equal_left_right_margin'] = [
        '#type' => 'select',
        '#title' => $this->t('Left Right Margins'),
        '#default_value' => $this->configuration['equal_left_right_margin'],
        '#options' => $this->getEqualLeftRightMarginsOptions(),
        '#description' => $this->t('Choose the left/right margins for this layout.'),
      ];
    }

    if (!empty($this->getEqualTopBottomPaddingsOptions())) {
      $form['equal_top_bottom_padding'] = [
        '#type' => 'select',
        '#title' => $this->t('Top Bottom Paddings'),
        '#default_value' => $this->configuration['equal_top_bottom_padding'],
        '#options' => $this->getEqualTopBottomPaddingsOptions(),
        '#description' => $this->t('Choose the top/bottom paddings for this layout.'),
      ];
    }

    if (!empty($this->getEqualLeftRightPaddingsOptions())) {
      $form['equal_left_right_padding'] = [
        '#type' => 'select',
        '#title' => $this->t('Left Right Paddings'),
        '#default_value' => $this->configuration['equal_left_right_padding'],
        '#options' => $this->getEqualLeftRightPaddingsOptions(),
        '#description' => $this->t('Choose the left/right paddings for this layout.'),
      ];
    }

    if (!empty($this->getContainerOptions())) {
      $form['container'] = [
        '#type' => 'select',
        '#title' => $this->t('Container'),
        '#default_value' => $this->configuration['container'],
        '#options' => $this->getContainerOptions(),
        '#description' => $this->t('Choose the size of the container for this layout.'),
      ];
    }

    if (!empty($this->getColorsOptions())) {
      $form['color'] = [
        '#type' => 'select',
        '#title' => $this->t('Default color'),
        '#default_value' => $this->configuration['color'],
        '#options' => $this->getColorsOptions(),
        '#description' => $this->t('Choose the default color for the text of this layout.'),
      ];
    }

    if (!empty($this->getModifiersOptions())) {
      $form['modifiers'] = [
        '#type' => 'select',
        '#title' => $this->t('Modifiers'),
        '#default_value' => $this->configuration['modifiers'],
        '#options' => $this->getModifiersOptions(),
        '#description' => $this->t('Choose the modifiers for this layout.'),
      ];
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);

    $background_file = $form_state->getValue('background_image', 0);
    if (isset($background_file[0]) && !empty($background_file[0])) {
      $file = File::load($background_file[0]);
      $file->setPermanent();
      $file->save();
    }

    $this->configuration['background_image'] = !empty($file) ? $file->id() : '';
    $this->configuration['background'] = $form_state->getValue('background');
    $this->configuration['background_attachment'] = $form_state->getValue('background_attachment');
    $this->configuration['background_position'] = $form_state->getValue('background_position');
    $this->configuration['background_size'] = $form_state->getValue('background_size');
    $this->configuration['background_overlay'] = $form_state->getValue('background_overlay');
    $this->configuration['equal_top_bottom_margin'] = $form_state->getValue('equal_top_bottom_margin');
    $this->configuration['equal_left_right_margin'] = $form_state->getValue('equal_left_right_margin');
    $this->configuration['equal_top_bottom_padding'] = $form_state->getValue('equal_top_bottom_padding');
    $this->configuration['equal_left_right_padding'] = $form_state->getValue('equal_left_right_padding');
    $this->configuration['container'] = $form_state->getValue('container');
    $this->configuration['color'] = $form_state->getValue('color');
    $this->configuration['column_gap'] = $form_state->getValue('column_gap');
    $this->configuration['row_gap'] = $form_state->getValue('row_gap');
    $this->configuration['column_width'] = $form_state->getValue('column_width');
    $this->configuration['column_breakpoint'] = $form_state->getValue('column_breakpoint');
    $this->configuration['modifiers'] = $form_state->getValue('modifiers');
  }

  /**
   * {@inheritdoc}
   */
  public function build(array $regions) {
    $build = parent::build($regions);
    $classes = [
      $this->configuration['background'],
      $this->configuration['background_attachment'],
      $this->configuration['background_position'],
      $this->configuration['background_size'],
      $this->configuration['background_overlay'],
      $this->configuration['equal_top_bottom_margin'],
      $this->configuration['equal_left_right_margin'],
      $this->configuration['equal_top_bottom_padding'],
      $this->configuration['equal_left_right_padding'],
      $this->configuration['container'],
      $this->configuration['color'],
      $this->configuration['column_gap'],
      $this->configuration['row_gap'],
      $this->configuration['column_breakpoint'],
      $this->configuration['column_width'],
      $this->configuration['modifiers'],
    ];
    $build_classes = array_merge($build['#attributes']['class'], $classes);
    $build['#attributes']['class'] = $build_classes;

    if (!empty($this->configuration['background_image'])) {
      $image = File::load($this->configuration['background_image']);
      if (!empty($image)) {
        $url = file_create_url($image->uri->value);
        $build['#attributes']['style'] = 'background-image: url("' . $url . '")';
      }
    }

    return $build;
  }

  /**
   * Default behavior to get the default config option.
   *
   * @return string
   *   Return the first option.
   */
  protected function getDefaultConfigOption($options) {
    return !empty($options) ? array_shift($options) : '';
  }

  /**
   * Get the default background option.
   *
   * @return string
   *   Return the default background option.
   */
  protected function getDefaultBackground() {
    $options = $this->getBackgroundOptions();
    return $this->getDefaultConfigOption($options);
  }

  /**
   * Get the default background option.
   *
   * @return string
   *   Return the default background option.
   */
  protected function getDefaultImageBackground() {
    return '';
  }

  /**
   * Get the default background attachment option.
   *
   * @return string
   *   Return the default background attachment option.
   */
  protected function getDefaultBackgroundAttachment() {
    $options = $this->getBackgroundAttachmentOptions();
    return $this->getDefaultConfigOption($options);
  }

  /**
   * Get the default background position option.
   *
   * @return string
   *   Return the default background position option.
   */
  protected function getDefaultBackgroundPosition() {
    $options = $this->getBackgroundPositionOptions();
    return $this->getDefaultConfigOption($options);
  }

  /**
   * Get the default background size option.
   *
   * @return string
   *   Return the default background size option.
   */
  protected function getDefaultBackgroundSize() {
    $options = $this->getBackgroundSizeOptions();
    return $this->getDefaultConfigOption($options);
  }

  /**
   * Get the default background overlay option.
   *
   * @return string
   *   Return the default background overlay option.
   */
  protected function getDefaultBackgroundOverlay() {
    $options = $this->getBackgroundOverlayOptions();
    return $this->getDefaultConfigOption($options);
  }

  /**
   * Get the default equal top bottom margin option.
   *
   * @return string
   *   Return the default equal top bottom margin option.
   */
  protected function getDefaultEqualTopBottomMargin() {
    $options = $this->getEqualTopBottomMarginsOptions();
    return $this->getDefaultConfigOption($options);
  }

  /**
   * Get the default equal left right margin option.
   *
   * @return string
   *   Return the default equal left right margin option.
   */
  protected function getDefaultEqualLeftRightMargin() {
    $options = $this->getEqualLeftRightMarginsOptions();
    return $this->getDefaultConfigOption($options);
  }

  /**
   * Get the default top margin option.
   *
   * @return string
   *   Return the default top margin option.
   */
  protected function getDefaultTopMargin() {
    $options = $this->getTopMarginOptions();
    return $this->getDefaultConfigOption($options);
  }

  /**
   * Get the default right margin option.
   *
   * @return string
   *   Return the default right margin option.
   */
  protected function getDefaultRightMargin() {
    $options = $this->getRightMarginOptions();
    return $this->getDefaultConfigOption($options);
  }

  /**
   * Get the default bottom margin option.
   *
   * @return string
   *   Return the default bottom margin option.
   */
  protected function getDefaultBottomMargin() {
    $options = $this->getBottomMarginOptions();
    return $this->getDefaultConfigOption($options);
  }

  /**
   * Get the default left margin option.
   *
   * @return string
   *   Return the default left margin option.
   */
  protected function getDefaultLeftMargin() {
    $options = $this->getLeftMarginOptions();
    return $this->getDefaultConfigOption($options);
  }

  /**
   * Get the default equal top bottom padding option.
   *
   * @return string
   *   Return the default equal top bottom padding option.
   */
  protected function getDefaultEqualTopBottomPadding() {
    $options = $this->getEqualTopBottomPaddingsOptions();
    return $this->getDefaultConfigOption($options);
  }

  /**
   * Get the default equal left right padding option.
   *
   * @return string
   *   Return the default equal left right padding option.
   */
  protected function getDefaultEqualLeftRightPadding() {
    $options = $this->getEqualLeftRightPaddingsOptions();
    return $this->getDefaultConfigOption($options);
  }

  /**
   * Get the default top padding option.
   *
   * @return string
   *   Return the default top padding option.
   */
  protected function getDefaultTopPadding() {
    $options = $this->getTopPaddingOptions();
    return $this->getDefaultConfigOption($options);
  }

  /**
   * Get the default right padding option.
   *
   * @return string
   *   Return the default right padding option.
   */
  protected function getDefaultRightPadding() {
    $options = $this->getRightPaddingOptions();
    return $this->getDefaultConfigOption($options);
  }

  /**
   * Get the default bottom padding option.
   *
   * @return string
   *   Return the default bottom padding option.
   */
  protected function getDefaultBottomPadding() {
    $options = $this->getBottomPaddingOptions();
    return $this->getDefaultConfigOption($options);
  }

  /**
   * Get the default left padding option.
   *
   * @return string
   *   Return the default left padding option.
   */
  protected function getDefaultLeftPadding() {
    $options = $this->getLeftPaddingOptions();
    return $this->getDefaultConfigOption($options);
  }

  /**
   * Get the default container option.
   *
   * @return string
   *   Return the default container option.
   */
  protected function getDefaultContainer() {
    $options = $this->getContainerOptions();
    return $this->getDefaultConfigOption($options);
  }

  /**
   * Get the default color option.
   *
   * @return string
   *   Return the default color option.
   */
  protected function getDefaultColor() {
    $options = $this->getColorsOptions();
    return $this->getDefaultConfigOption($options);
  }

  /**
   * Get the default column gap option.
   *
   * @return string
   *   Return the default column gap option.
   */
  protected function getDefaultColumnGap() {
    $options = $this->getColumnGapOptions();
    return $this->getDefaultConfigOption($options);
  }

  /**
   * Get the default row gap option.
   *
   * @return string
   *   Return the default row gap option.
   */
  protected function getDefaultRowGap() {
    $options = $this->getRowGapOptions();
    return $this->getDefaultConfigOption($options);
  }

  /**
   * Get the default column breakpoint option.
   *
   * @return string
   *   Return the default column breakpoint option.
   */
  protected function getDefaultColumnBreakpoint() {
    $options = $this->getColumnBreakpointOptions();
    return $this->getDefaultConfigOption($options);
  }

  /**
   * Get the default column width option.
   *
   * @return string
   *   Return the default column width option.
   */
  protected function getDefaultColumnWidth() {
    $options = $this->getColumnWidthOptions();
    return $this->getDefaultConfigOption($options);
  }

  /**
   * Get the default modifiers option.
   *
   * @return string
   *   Return the default modifiers option.
   */
  protected function getDefaultModifiers() {
    $options = $this->getModifiersOptions();
    return $this->getDefaultConfigOption($options);
  }

  /**
   * Function to enable the multiple columns features.
   *
   * Override this function and return TRUE in order to enable the multiple
   * columns features.
   *
   * @return bool
   *   Boolean to know if the multiple columns features are enabled or not.
   */
  protected function isMultipleColumnsLayout() {
    return FALSE;
  }

  /**
   * Function to enable the background image feature.
   *
   * Override this function and return TRUE in order to enable the image
   * background feature.
   *
   * @return bool
   *   Boolean to know if the image background feature is enabled or not.
   */
  protected function enableImageBackground() {
    return FALSE;
  }

  /**
   * Gets the background options for the configuration form.
   *
   * @return string[]
   *   The background options array where the keys are strings that will
   *   be added to the CSS classes and the values are the human readable labels.
   */
  abstract protected function getBackgroundOptions();

  /**
   * Gets the background attachment options for the configuration form.
   *
   * @return string[]
   *   The background attachment options array where the keys are strings that
   *   will be added to the CSS classes and the values are the human readable
   *   labels.
   */
  abstract protected function getBackgroundAttachmentOptions();

  /**
   * Gets the background position options for the configuration form.
   *
   * @return string[]
   *   The background position options array where the keys are strings that
   *   will be added to the CSS classes and the values are the human readable
   *   labels.
   */
  abstract protected function getBackgroundPositionOptions();

  /**
   * Gets the background size options for the configuration form.
   *
   * @return string[]
   *   The background size options array where the keys are strings that
   *   will be added to the CSS classes and the values are the human readable
   *   labels.
   */
  abstract protected function getBackgroundSizeOptions();

  /**
   * Gets the background overlay options for the configuration form.
   *
   * @return string[]
   *   The background overlay options array where the keys are strings that
   *   will be added to the CSS classes and the values are the human readable
   *   labels.
   */
  abstract protected function getBackgroundOverlayOptions();

  /**
   * Gets the top and bottom equal margin options for the configuration form.
   *
   * @return string[]
   *   The top and bottom equal margin options array where the keys are strings
   *   that will be added to the CSS classes and the values are the human
   *   readable labels.
   */
  abstract protected function getEqualTopBottomMarginsOptions();

  /**
   * Gets the left and right equal margin options for the configuration form.
   *
   * @return string[]
   *   The left and right equal margin options array where the keys are strings
   *   that will be added to the CSS classes and the values are the human
   *   readable labels.
   */
  abstract protected function getEqualLeftRightMarginsOptions();

  /**
   * Gets the top margin options for the configuration form.
   *
   * @return string[]
   *   The top margin options array where the keys are strings
   *   that will be added to the CSS classes and the values are the human
   *   readable labels.
   */
  abstract protected function getTopMarginOptions();

  /**
   * Gets the right margin options for the configuration form.
   *
   * @return string[]
   *   The right margin options array where the keys are strings
   *   that will be added to the CSS classes and the values are the human
   *   readable labels.
   */
  abstract protected function getRightMarginOptions();

  /**
   * Gets the bottom margin options for the configuration form.
   *
   * @return string[]
   *   The bottom margin options array where the keys are strings
   *   that will be added to the CSS classes and the values are the human
   *   readable labels.
   */
  abstract protected function getBottomMarginOptions();

  /**
   * Gets the left margin options for the configuration form.
   *
   * @return string[]
   *   The left margin options array where the keys are strings
   *   that will be added to the CSS classes and the values are the human
   *   readable labels.
   */
  abstract protected function getLeftMarginOptions();

  /**
   * Gets the top and bottom equal padding options for the configuration form.
   *
   * @return string[]
   *   The top and bottom equal padding options array where the keys are strings
   *   that will be added to the CSS classes and the values are the human
   *   readable labels.
   */
  abstract protected function getEqualTopBottomPaddingsOptions();

  /**
   * Gets the left and right equal padding options for the configuration form.
   *
   * @return string[]
   *   The left and right equal padding options array where the keys are strings
   *   that will be added to the CSS classes and the values are the human
   *   readable labels.
   */
  abstract protected function getEqualLeftRightPaddingsOptions();

  /**
   * Gets the top padding options for the configuration form.
   *
   * @return string[]
   *   The top padding options array where the keys are strings
   *   that will be added to the CSS classes and the values are the human
   *   readable labels.
   */
  abstract protected function getTopPaddingOptions();

  /**
   * Gets the right padding options for the configuration form.
   *
   * @return string[]
   *   The right padding options array where the keys are strings
   *   that will be added to the CSS classes and the values are the human
   *   readable labels.
   */
  abstract protected function getRightPaddingOptions();

  /**
   * Gets the bottom padding options for the configuration form.
   *
   * @return string[]
   *   The bottom padding options array where the keys are strings
   *   that will be added to the CSS classes and the values are the human
   *   readable labels.
   */
  abstract protected function getBottomPaddingOptions();

  /**
   * Gets the left padding options for the configuration form.
   *
   * @return string[]
   *   The left padding options array where the keys are strings
   *   that will be added to the CSS classes and the values are the human
   *   readable labels.
   */
  abstract protected function getLeftPaddingOptions();

  /**
   * Gets the container options for the configuration form.
   *
   * @return string[]
   *   The container options array where the keys are strings that will be added
   *   to the CSS classes and the values are the human readable labels.
   */
  abstract protected function getContainerOptions();

  /**
   * Gets the colors options for the configuration form.
   *
   * @return string[]
   *   The colors options array where the keys are strings that will be added to
   *   the CSS classes and the values are the human readable labels.
   */
  abstract protected function getColorsOptions();

  /**
   * Gets the column gap options for the configuration form.
   *
   * @return string[]
   *   The column gap options array where the keys are strings that will be
   *   added to the CSS classes and the values are the human readable labels.
   */
  abstract protected function getColumnGapOptions();

  /**
   * Gets the row gap options for the configuration form.
   *
   * @return string[]
   *   The row gap options array where the keys are strings that will be
   *   added to the CSS classes and the values are the human readable labels.
   */
  abstract protected function getRowGapOptions();

  /**
   * Gets the column breakpoint options for the configuration form.
   *
   * @return string[]
   *   The column breakpoint options array where the keys are strings that will
   *   be added to the CSS classes and the values are the human readable labels.
   */
  abstract protected function getColumnBreakpointOptions();

  /**
   * Gets the column width options for the configuration form.
   *
   * @return string[]
   *   The column width options array where the keys are strings that will
   *   be added to the CSS classes and the values are the human readable labels.
   */
  abstract protected function getColumnWidthOptions();

  /**
   * Gets the modifiers options for the configuration form.
   *
   * @return string[]
   *   The modifiers options array where the keys are strings that will
   *   be added to the CSS classes and the values are the human readable labels.
   *   The modifiers are there for developers in order to implement special
   *   logic.
   */
  abstract protected function getModifiersOptions();

}
